@@ -81,6 +81,7 @@ pub enum Field {
     IsAudio,
     IsBook,
     IsDoc,
+    IsFont,
     IsImage,
     IsSource,
     IsVideo,
@@ -171,6 +172,7 @@ impl FromStr for Field {
             "is_audio" => Ok(Field::IsAudio),
             "is_book" => Ok(Field::IsBook),
             "is_doc" => Ok(Field::IsDoc),
+            "is_font" => Ok(Field::IsFont),
             "is_image" => Ok(Field::IsImage),
             "is_source" => Ok(Field::IsSource),
             "is_video" => Ok(Field::IsVideo),
@@ -255,6 +257,7 @@ impl Field {
             | Field::IsAudio
             | Field::IsBook
             | Field::IsDoc
+            | Field::IsFont
             | Field::IsImage
             | Field::IsSource
             | Field::IsVideo => true,
@@ -301,6 +304,7 @@ impl Field {
             | Field::IsAudio
             | Field::IsBook
             | Field::IsDoc
+            | Field::IsFont
             | Field::IsImage
             | Field::IsSource
             | Field::IsVideo
