@@ -1,55 +1,151 @@
 use std::collections::{HashMap, HashSet, VecDeque};
 use std::fs;
-use std::fs::{DirEntry, FileType};
-use std::fs::Metadata;
+use std::fs::{DirEntry, FileType, Metadata};
 #[cfg(unix)]
 use std::fs::symlink_metadata;
 use std::io;
-use std::io::ErrorKind;
-use std::io::Write;
+use std::io::{ErrorKind, Write};
 use std::ops::Add;
 #[cfg(unix)]
-use std::os::unix::fs::DirEntryExt;
-#[cfg(unix)]
-use std::os::unix::fs::MetadataExt;
-use std::path::Path;
-use std::path::PathBuf;
+use std::os::unix::fs::{DirEntryExt, MetadataExt};
+use std::path::{Path, PathBuf};
+use std::rc::Rc;
 
 use chrono::{DateTime, Local};
 use lscolors::{LsColors, Style};
-use mp3_metadata;
 use mp3_metadata::MP3Metadata;
 use regex::Regex;
 #[cfg(all(unix, feature = "users"))]
-use users::{Groups, Users, UsersCache};
+use uzers::{Groups, Users, UsersCache};
 #[cfg(unix)]
 use xattr::FileExt;
-use zip;
 
 use crate::config::Config;
 use crate::expr::Expr;
 use crate::field::Field;
-use crate::fileinfo::FileInfo;
-use crate::fileinfo::to_file_info;
+use crate::fileinfo::{FileInfo, to_file_info};
 use crate::function;
-use crate::function::Variant;
-use crate::function::VariantType;
-use crate::ignore::docker::{DockerignoreFilter, search_upstream_dockerignore};
-use crate::ignore::docker::matches_dockerignore_filter;
-use crate::ignore::git::{get_gitignore_filters, GitignoreFilter, search_upstream_gitignore, update_gitignore_map};
-use crate::ignore::git::matches_gitignore_filter;
-use crate::ignore::hg::{HgignoreFilter, search_upstream_hgignore};
-use crate::ignore::hg::matches_hgignore_filter;
+use crate::function::{Variant, VariantType};
+use crate::ignore::docker::{DockerignoreFilter, matches_dockerignore_filter, search_upstream_dockerignore};
+use crate::ignore::git::{get_gitignore_filters, GitignoreFilter, matches_gitignore_filter, search_upstream_gitignore, update_gitignore_map};
+use crate::ignore::hg::{HgignoreFilter, matches_hgignore_filter, search_upstream_hgignore};
 use crate::mode;
-use crate::operators::LogicalOp;
-use crate::operators::Op;
+use crate::operators::{LogicalOp, Op};
+use crate::output::ResultsWriter;
 use crate::query::{Query, Root, TraversalMode};
 use crate::query::TraversalMode::Bfs;
 use crate::util::*;
 use crate::util::dimensions::get_dimensions;
-use crate::output::ResultsWriter;
 use crate::util::duration::get_duration;
 
+struct FileMetadataState {
+    file_metadata_set: bool,
+    file_metadata: Option<Metadata>,
+
+    line_count_set: bool,
+    line_count: Option<usize>,
+
+    dimensions_set: bool,
+    dimensions: Option<Dimensions>,
+
+    duration_set: bool,
+    duration: Option<Duration>,
+
+    mp3_metadata_set: bool,
+    mp3_metadata: Option<MP3Metadata>,
+
+    exif_metadata_set: bool,
+    exif_metadata: Option<HashMap<String, String>>,
+}
+
+impl FileMetadataState {
+    fn new() -> FileMetadataState {
+        FileMetadataState {
+            file_metadata_set: false,
+            file_metadata: None,
+
+            line_count_set: false,
+            line_count: None,
+
+            dimensions_set: false,
+            dimensions: None,
+
+            duration_set: false,
+            duration: None,
+
+            mp3_metadata_set: false,
+            mp3_metadata: None,
+
+            exif_metadata_set: false,
+            exif_metadata: None,
+        }
+    }
+
+    fn clear(&mut self) {
+        self.file_metadata_set = false;
+        self.file_metadata = None;
+
+        self.line_count_set = false;
+        self.line_count = None;
+
+        self.dimensions_set = false;
+        self.dimensions = None;
+
+        self.duration_set = false;
+        self.duration = None;
+
+        self.mp3_metadata_set = false;
+        self.mp3_metadata = None;
+
+        self.exif_metadata_set = false;
+        self.exif_metadata = None;
+    }
+
+    fn update_file_metadata(&mut self, entry: &DirEntry, follow_symlinks: bool) {
+        if !self.file_metadata_set {
+            self.file_metadata_set = true;
+            self.file_metadata = get_metadata(entry, follow_symlinks);
+        }
+    }
+
+    fn update_line_count(&mut self, entry: &DirEntry) {
+        if !self.line_count_set {
+            self.line_count_set = true;
+            self.line_count = get_line_count(entry);
+        }
+    }
+
+    fn update_mp3_metadata(&mut self, entry: &DirEntry) {
+        if !self.mp3_metadata_set {
+            self.mp3_metadata_set = true;
+            self.mp3_metadata = get_mp3_metadata(entry);
+        }
+    }
+
+    fn update_exif_metadata(&mut self, entry: &DirEntry) {
+        if !self.exif_metadata_set {
+            self.exif_metadata_set = true;
+            self.exif_metadata = get_exif_metadata(entry);
+        }
+    }
+
+    fn update_dimensions(&mut self, entry: &DirEntry) {
+        if !self.dimensions_set {
+            self.dimensions_set = true;
+            self.dimensions = get_dimensions(entry.path());
+        }
+    }
+
+    fn update_duration(&mut self, entry: &DirEntry) {
+        if !self.duration_set {
+            self.update_mp3_metadata(entry);
+
+            self.duration_set = true;
+            self.duration = get_duration(entry.path(), &self.mp3_metadata);
+        }
+    }
+}
+
 pub struct Searcher<'a> {
     query: &'a Query,
     config : &'a Config,
@@ -60,7 +156,7 @@ pub struct Searcher<'a> {
     regex_cache: HashMap<String, Regex>,
     found: u32,
     raw_output_buffer: Vec<HashMap<String, String>>,
-    partitioned_output_buffer: HashMap<Vec<String>, Vec<HashMap<String, String>>>,
+    partitioned_output_buffer: Rc<HashMap<Vec<String>, Vec<HashMap<String, String>>>>,
     output_buffer: TopN<Criteria<String>, String>,
     gitignore_map: HashMap<PathBuf, Vec<GitignoreFilter>>,
     hgignore_filters: Vec<HgignoreFilter>,
@@ -72,23 +168,7 @@ pub struct Searcher<'a> {
     dir_queue: Box<VecDeque<PathBuf>>,
     current_follow_symlinks: bool,
 
-    file_metadata: Option<Metadata>,
-    file_metadata_set: bool,
-
-    file_line_count: Option<usize>,
-    file_line_count_set: bool,
-
-    file_dimensions: Option<Dimensions>,
-    file_dimensions_set: bool,
-
-    file_duration: Option<crate::util::Duration>,
-    file_duration_set: bool,
-
-    file_mp3_metadata: Option<MP3Metadata>,
-    file_mp3_metadata_set: bool,
-
-    file_exif_metadata: Option<HashMap<String, String>>,
-    file_exif_metadata_set: bool,
+    fms: FileMetadataState,
 
     pub error_count: i32,
 }
@@ -108,7 +188,7 @@ impl <'a> Searcher<'a> {
             regex_cache: HashMap::new(),
             found: 0,
             raw_output_buffer: vec![],
-            partitioned_output_buffer: HashMap::new(),
+            partitioned_output_buffer: Rc::new(HashMap::new()),
             output_buffer: if limit == 0 { TopN::limitless() } else { TopN::new(limit) },
             gitignore_map: HashMap::new(),
             hgignore_filters: vec![],
@@ -120,48 +200,12 @@ impl <'a> Searcher<'a> {
             dir_queue: Box::from(VecDeque::new()),
             current_follow_symlinks: false,
 
-            file_metadata: None,
-            file_metadata_set: false,
-
-            file_line_count: None,
-            file_line_count_set: false,
-
-            file_dimensions: None,
-            file_dimensions_set: false,
-
-            file_duration: None,
-            file_duration_set: false,
-
-            file_mp3_metadata: None,
-            file_mp3_metadata_set: false,
-
-            file_exif_metadata: None,
-            file_exif_metadata_set: false,
+            fms: FileMetadataState::new(),
 
             error_count: 0,
         }
     }
 
-    fn clear_file_data(&mut self) {
-        self.file_metadata_set = false;
-        self.file_metadata = None;
-
-        self.file_line_count = None;
-        self.file_line_count_set = false;
-
-        self.file_dimensions_set = false;
-        self.file_dimensions = None;
-
-        self.file_duration_set = false;
-        self.file_duration = None;
-
-        self.file_mp3_metadata_set = false;
-        self.file_mp3_metadata = None;
-
-        self.file_exif_metadata_set = false;
-        self.file_exif_metadata = None;
-    }
-
     pub fn is_buffered(&self) -> bool {
         self.has_ordering() || self.has_aggregate_column()
     }
@@ -185,7 +229,7 @@ impl <'a> Searcher<'a> {
 
         let mut roots = vec![];
 
-        for root in self.query.roots.clone() {
+        for root in &self.query.roots {
             if root.options.regexp {
                 let mut ext_roots: Vec<String> = vec![];
                 let parts = root.path.split('/').collect::<Vec<&str>>();
@@ -204,7 +248,7 @@ impl <'a> Searcher<'a> {
                             }
                         }
 
-                        for root in ext_roots.clone() {
+                        for root in &ext_roots {
                             let mut start_from_rx_dir = false;
 
                             let mut path = Path::new(&root);
@@ -255,7 +299,7 @@ impl <'a> Searcher<'a> {
 
                 ext_roots.iter().for_each(|ext_root| roots.push(Root::clone_with_path(ext_root.to_string(), root.clone())));
             } else {
-                roots.push(root);
+                roots.push(root.clone());
             }
         }
 
@@ -313,7 +357,7 @@ impl <'a> Searcher<'a> {
         if self.has_aggregate_column() {
             if !self.query.grouping_fields.is_empty() {
                 if self.partitioned_output_buffer.is_empty() {
-                    self.partitioned_output_buffer = self.partition_output_buffer();
+                    self.partitioned_output_buffer = Rc::new(self.partition_output_buffer());
                 }
 
                 let group_keys: Vec<String> = self.query.grouping_fields.iter().map(|f| f.to_string()).collect();
@@ -324,11 +368,11 @@ impl <'a> Searcher<'a> {
                     let mut items: Vec<(String, String)> = Vec::new();
 
                     let mut file_map = HashMap::new();
-                    for (i, k) in group_keys.clone().into_iter().enumerate() {
-                        file_map.insert(k, f.0.get(i).unwrap().clone());
+                    for (i, k) in group_keys.iter().enumerate() {
+                        file_map.insert(k.clone(), f.0.get(i).unwrap().clone());
                     }
 
-                    for column_expr in &self.query.fields.clone() {
+                    for column_expr in &self.query.fields {
                         let record = format!("{}", self.get_column_expr_value(None, &None, &mut file_map, Some(f.1), column_expr));
                         let field_name = column_expr.to_string().to_lowercase();
                         items.push((field_name, record));
@@ -342,7 +386,7 @@ impl <'a> Searcher<'a> {
                 let mut buf = WritableBuffer::new();
                 let mut items: Vec<(String, String)> = Vec::new();
 
-                for column_expr in &self.query.fields.clone() {
+                for column_expr in &self.query.fields {
                     let record = format!("{}", self.get_column_expr_value(None, &None, &mut HashMap::new(), None, column_expr));
                     let field_name = column_expr.to_string().to_lowercase();
                     items.push((field_name, record));
@@ -695,34 +739,6 @@ impl <'a> Searcher<'a> {
         result
     }
 
-    fn update_file_metadata(&mut self, entry: &DirEntry) {
-        if !self.file_metadata_set {
-            self.file_metadata_set = true;
-            self.file_metadata = get_metadata(entry, self.current_follow_symlinks);
-        }
-    }
-
-    fn update_file_line_count(&mut self, entry: &DirEntry) {
-        if !self.file_line_count_set {
-            self.file_line_count_set = true;
-            self.file_line_count = crate::util::get_line_count(entry);
-        }
-    }
-
-    fn update_file_mp3_metadata(&mut self, entry: &DirEntry) {
-        if !self.file_mp3_metadata_set {
-            self.file_mp3_metadata_set = true;
-            self.file_mp3_metadata = get_mp3_metadata(entry);
-        }
-    }
-
-    fn update_file_exif_metadata(&mut self, entry: &DirEntry) {
-        if !self.file_exif_metadata_set {
-            self.file_exif_metadata_set = true;
-            self.file_exif_metadata = get_exif_metadata(entry);
-        }
-    }
-
     fn get_field_value(&mut self,
                        entry: &DirEntry,
                        file_info: &Option<FileInfo>,
@@ -806,9 +822,9 @@ impl <'a> Searcher<'a> {
                         return Variant::from_int(file_info.size as i64);
                     },
                     _ => {
-                        self.update_file_metadata(entry);
+                        self.fms.update_file_metadata(entry, self.current_follow_symlinks);
 
-                        if let Some(ref attrs) = self.file_metadata {
+                        if let Some(ref attrs) = self.fms.file_metadata {
                             return Variant::from_int(attrs.len() as i64);
                         }
                     }
@@ -820,9 +836,9 @@ impl <'a> Searcher<'a> {
                         return Variant::from_string( &format_filesize(file_info.size, self.config.default_file_size_format.as_ref().unwrap_or(&String::new())));
                     },
                     _ => {
-                        self.update_file_metadata(entry);
+                        self.fms.update_file_metadata(entry, self.current_follow_symlinks);
 
-                        if let Some(ref attrs) = self.file_metadata {
+                        if let Some(ref attrs) = self.fms.file_metadata {
                             return Variant::from_string(&format_filesize(attrs.len(), self.config.default_file_size_format.as_ref().unwrap_or(&String::new())));
                         }
                     }
@@ -834,9 +850,9 @@ impl <'a> Searcher<'a> {
                         return Variant::from_bool(file_info.name.ends_with('/') || file_info.name.ends_with('\\'));
                     },
                     _ => {
-                        self.update_file_metadata(entry);
+                        self.fms.update_file_metadata(entry, self.current_follow_symlinks);
 
-                        if let Some(ref attrs) = self.file_metadata {
+                        if let Some(ref attrs) = self.fms.file_metadata {
                             return Variant::from_bool(attrs.is_dir());
                         }
                     }
@@ -848,9 +864,9 @@ impl <'a> Searcher<'a> {
                         return Variant::from_bool(!file_info.name.ends_with('/'));
                     },
                     _ => {
-                        self.update_file_metadata(entry);
+                        self.fms.update_file_metadata(entry, self.current_follow_symlinks);
 
-                        if let Some(ref attrs) = self.file_metadata {
+                        if let Some(ref attrs) = self.fms.file_metadata {
                             return Variant::from_bool(attrs.is_file());
                         }
                     }
@@ -862,9 +878,9 @@ impl <'a> Searcher<'a> {
                         return Variant::from_bool(false);
                     },
                     _ => {
-                        self.update_file_metadata(entry);
+                        self.fms.update_file_metadata(entry, self.current_follow_symlinks);
 
-                        if let Some(ref attrs) = self.file_metadata {
+                        if let Some(ref attrs) = self.fms.file_metadata {
                             return Variant::from_bool(attrs.file_type().is_symlink());
                         }
                     }
@@ -885,9 +901,9 @@ impl <'a> Searcher<'a> {
             Field::Device => {
                 #[cfg(unix)]
                     {
-                        self.update_file_metadata(entry);
+                        self.fms.update_file_metadata(entry, self.current_follow_symlinks);
 
-                        if let Some(ref attrs) = self.file_metadata {
+                        if let Some(ref attrs) = self.fms.file_metadata {
                             return Variant::from_int(attrs.dev() as i64);
                         }
                     }
@@ -897,9 +913,9 @@ impl <'a> Searcher<'a> {
             Field::Inode => {
                 #[cfg(unix)]
                     {
-                        self.update_file_metadata(entry);
+                        self.fms.update_file_metadata(entry, self.current_follow_symlinks);
 
-                        if let Some(ref attrs) = self.file_metadata {
+                        if let Some(ref attrs) = self.fms.file_metadata {
                             return Variant::from_int(attrs.ino() as i64);
                         }
                     }
@@ -909,9 +925,9 @@ impl <'a> Searcher<'a> {
             Field::Blocks => {
                 #[cfg(unix)]
                     {
-                        self.update_file_metadata(entry);
+                        self.fms.update_file_metadata(entry, self.current_follow_symlinks);
 
-                        if let Some(ref attrs) = self.file_metadata {
+                        if let Some(ref attrs) = self.fms.file_metadata {
                             return Variant::from_int(attrs.blocks() as i64);
                         }
                     }
@@ -921,9 +937,9 @@ impl <'a> Searcher<'a> {
             Field::Hardlinks => {
                 #[cfg(unix)]
                     {
-                        self.update_file_metadata(entry);
+                        self.fms.update_file_metadata(entry, self.current_follow_symlinks);
 
-                        if let Some(ref attrs) = self.file_metadata {
+                        if let Some(ref attrs) = self.fms.file_metadata {
                             return Variant::from_int(attrs.nlink() as i64);
                         }
                     }
@@ -938,9 +954,9 @@ impl <'a> Searcher<'a> {
                         }
                     },
                     _ => {
-                        self.update_file_metadata(entry);
+                        self.fms.update_file_metadata(entry, self.current_follow_symlinks);
 
-                        if let Some(ref attrs) = self.file_metadata {
+                        if let Some(ref attrs) = self.fms.file_metadata {
                             return Variant::from_string(&mode::get_mode(attrs));
                         }
                     }
@@ -994,25 +1010,25 @@ impl <'a> Searcher<'a> {
                         return Variant::from_bool(is_hidden(&file_info.name, &None, true));
                     },
                     _ => {
-                        self.update_file_metadata(entry);
+                        self.fms.update_file_metadata(entry, self.current_follow_symlinks);
 
-                        return Variant::from_bool(is_hidden(&entry.file_name().to_string_lossy(), &self.file_metadata, false));
+                        return Variant::from_bool(is_hidden(&entry.file_name().to_string_lossy(), &self.fms.file_metadata, false));
                     }
                 }
             },
             Field::Uid => {
-                self.update_file_metadata(entry);
+                self.fms.update_file_metadata(entry, self.current_follow_symlinks);
 
-                if let Some(ref attrs) = self.file_metadata {
+                if let Some(ref attrs) = self.fms.file_metadata {
                     if let Some(uid) = mode::get_uid(attrs) {
                         return Variant::from_int(uid as i64);
                     }
                 }
             },
             Field::Gid => {
-                self.update_file_metadata(entry);
+                self.fms.update_file_metadata(entry, self.current_follow_symlinks);
 
-                if let Some(ref attrs) = self.file_metadata {
+                if let Some(ref attrs) = self.fms.file_metadata {
                     if let Some(gid) = mode::get_gid(attrs) {
                         return Variant::from_int(gid as i64);
                     }
@@ -1020,9 +1036,9 @@ impl <'a> Searcher<'a> {
             },
             #[cfg(all(unix, feature = "users"))]
             Field::User => {
-                self.update_file_metadata(entry);
+                self.fms.update_file_metadata(entry, self.current_follow_symlinks);
 
-                if let Some(ref attrs) = self.file_metadata {
+                if let Some(ref attrs) = self.fms.file_metadata {
                     if let Some(uid) = mode::get_uid(attrs) {
                         if let Some(user) = self.user_cache.get_user_by_uid(uid) {
                             return Variant::from_string(&user.name().to_string_lossy().to_string());
@@ -1032,9 +1048,9 @@ impl <'a> Searcher<'a> {
             },
             #[cfg(all(unix, feature = "users"))]
             Field::Group => {
-                self.update_file_metadata(entry);
+                self.fms.update_file_metadata(entry, self.current_follow_symlinks);
 
-                if let Some(ref attrs) = self.file_metadata {
+                if let Some(ref attrs) = self.fms.file_metadata {
                     if let Some(gid) = mode::get_gid(attrs) {
                         if let Some(group) = self.user_cache.get_group_by_gid(gid) {
                             return Variant::from_string(&group.name().to_string_lossy().to_string());
@@ -1043,9 +1059,9 @@ impl <'a> Searcher<'a> {
                 }
             },
             Field::Created => {
-                self.update_file_metadata(entry);
+                self.fms.update_file_metadata(entry, self.current_follow_symlinks);
 
-                if let Some(ref attrs) = self.file_metadata {
+                if let Some(ref attrs) = self.fms.file_metadata {
                     if let Ok(sdt) = attrs.created() {
                         let dt: DateTime<Local> = DateTime::from(sdt);
                         return Variant::from_datetime(dt.naive_local());
@@ -1053,9 +1069,9 @@ impl <'a> Searcher<'a> {
                 }
             },
             Field::Accessed => {
-                self.update_file_metadata(entry);
+                self.fms.update_file_metadata(entry, self.current_follow_symlinks);
 
-                if let Some(ref attrs) = self.file_metadata {
+                if let Some(ref attrs) = self.fms.file_metadata {
                     if let Ok(sdt) = attrs.accessed() {
                         let dt: DateTime<Local> = DateTime::from(sdt);
                         return Variant::from_datetime(dt.naive_local());
@@ -1069,9 +1085,9 @@ impl <'a> Searcher<'a> {
                         return Variant::from_datetime(dt);
                     },
                     _ => {
-                        self.update_file_metadata(entry);
+                        self.fms.update_file_metadata(entry, self.current_follow_symlinks);
 
-                        if let Some(ref attrs) = self.file_metadata {
+                        if let Some(ref attrs) = self.fms.file_metadata {
                             if let Ok(sdt) = attrs.modified() {
                                 let dt: DateTime<Local> = DateTime::from(sdt);
                                 return Variant::from_datetime(dt.naive_local());
@@ -1118,9 +1134,9 @@ impl <'a> Searcher<'a> {
                         return Variant::from_bool(file_info.size == 0);
                     },
                     _ => {
-                        self.update_file_metadata(entry);
+                        self.fms.update_file_metadata(entry, self.current_follow_symlinks);
 
-                        if let Some(ref attrs) = self.file_metadata {
+                        if let Some(ref attrs) = self.fms.file_metadata {
                             return match attrs.is_dir() {
                                 true =>  match is_dir_empty(entry) {
                                     Some(result) => Variant::from_bool(result),
@@ -1133,100 +1149,89 @@ impl <'a> Searcher<'a> {
                 }
             },
             Field::Width => {
-                if !self.file_dimensions_set {
-                    self.file_dimensions_set = true;
-                    self.file_dimensions = get_dimensions(entry.path());
-                }
+                self.fms.update_dimensions(entry);
 
-                if let Some(Dimensions { width, .. }) = self.file_dimensions {
+                if let Some(Dimensions { width, .. }) = self.fms.dimensions {
                     return Variant::from_int(width as i64);
                 }
             },
             Field::Height => {
-                if !self.file_dimensions_set {
-                    self.file_dimensions_set = true;
-                    self.file_dimensions = get_dimensions(entry.path());
-                }
+                self.fms.update_dimensions(entry);
 
-                if let Some(Dimensions { height, .. }) = self.file_dimensions {
+                if let Some(Dimensions { height, .. }) = self.fms.dimensions {
                     return Variant::from_int(height as i64);
                 }
             },
             Field::Duration => {
-                if !self.file_duration_set {
-                    self.update_file_mp3_metadata(entry);
+                self.fms.update_duration(entry);
 
-                    self.file_duration_set = true;
-                    self.file_duration = get_duration(entry.path(), &self.file_mp3_metadata);
-                }
-
-                if let Some(Duration { length, .. }) = self.file_duration {
+                if let Some(Duration { length, .. }) = self.fms.duration {
                     return Variant::from_int(length as i64);
                 }
             },
             Field::Bitrate => {
-                self.update_file_mp3_metadata(entry);
+                self.fms.update_mp3_metadata(entry);
 
-                if let Some(ref mp3_info) = self.file_mp3_metadata {
+                if let Some(ref mp3_info) = self.fms.mp3_metadata {
                     return Variant::from_int(mp3_info.frames[0].bitrate as i64);
                 }
             },
             Field::Freq => {
-                self.update_file_mp3_metadata(entry);
+                self.fms.update_mp3_metadata(entry);
 
-                if let Some(ref mp3_info) = self.file_mp3_metadata {
+                if let Some(ref mp3_info) = self.fms.mp3_metadata {
                     return Variant::from_int(mp3_info.frames[0].sampling_freq as i64);
                 }
             },
             Field::Title => {
-                self.update_file_mp3_metadata(entry);
+                self.fms.update_mp3_metadata(entry);
 
-                if let Some(ref mp3_info) = self.file_mp3_metadata {
+                if let Some(ref mp3_info) = self.fms.mp3_metadata {
                     if let Some(ref mp3_tag) = mp3_info.tag {
                         return Variant::from_string(&mp3_tag.title);
                     }
                 }
             },
             Field::Artist => {
-                self.update_file_mp3_metadata(entry);
+                self.fms.update_mp3_metadata(entry);
 
-                if let Some(ref mp3_info) = self.file_mp3_metadata {
+                if let Some(ref mp3_info) = self.fms.mp3_metadata {
                     if let Some(ref mp3_tag) = mp3_info.tag {
                         return Variant::from_string(&mp3_tag.artist);
                     }
                 }
             },
             Field::Album => {
-                self.update_file_mp3_metadata(entry);
+                self.fms.update_mp3_metadata(entry);
 
-                if let Some(ref mp3_info) = self.file_mp3_metadata {
+                if let Some(ref mp3_info) = self.fms.mp3_metadata {
                     if let Some(ref mp3_tag) = mp3_info.tag {
                         return Variant::from_string(&mp3_tag.album);
                     }
                 }
             },
             Field::Year => {
-                self.update_file_mp3_metadata(entry);
+                self.fms.update_mp3_metadata(entry);
 
-                if let Some(ref mp3_info) = self.file_mp3_metadata {
+                if let Some(ref mp3_info) = self.fms.mp3_metadata {
                     if let Some(ref mp3_tag) = mp3_info.tag {
                         return Variant::from_int(mp3_tag.year as i64);
                     }
                 }
             },
             Field::Genre => {
-                self.update_file_mp3_metadata(entry);
+                self.fms.update_mp3_metadata(entry);
 
-                if let Some(ref mp3_info) = self.file_mp3_metadata {
+                if let Some(ref mp3_info) = self.fms.mp3_metadata {
                     if let Some(ref mp3_tag) = mp3_info.tag {
                         return Variant::from_string(&format!("{:?}", mp3_tag.genre));
                     }
                 }
             },
             Field::ExifDateTime => {
-                self.update_file_exif_metadata(entry);
+                self.fms.update_exif_metadata(entry);
 
-                if let Some(ref exif_info) = self.file_exif_metadata {
+                if let Some(ref exif_info) = self.fms.exif_metadata {
                     if let Some(exif_value) = exif_info.get("DateTime") {
                         if let Ok(exif_datetime) = parse_datetime(&exif_value) {
                             return Variant::from_datetime(exif_datetime.0);
@@ -1235,72 +1240,72 @@ impl <'a> Searcher<'a> {
                 }
             },
             Field::ExifGpsAltitude => {
-                self.update_file_exif_metadata(entry);
+                self.fms.update_exif_metadata(entry);
 
-                if let Some(ref exif_info) = self.file_exif_metadata {
+                if let Some(ref exif_info) = self.fms.exif_metadata {
                     if let Some(exif_value) = exif_info.get("__Alt") {
                         return Variant::from_float(exif_value.parse().unwrap_or(0.0));
                     }
                 }
             },
             Field::ExifGpsLatitude => {
-                self.update_file_exif_metadata(entry);
+                self.fms.update_exif_metadata(entry);
 
-                if let Some(ref exif_info) = self.file_exif_metadata {
+                if let Some(ref exif_info) = self.fms.exif_metadata {
                     if let Some(exif_value) = exif_info.get("__Lat") {
                         return Variant::from_float(exif_value.parse().unwrap_or(0.0));
                     }
                 }
             },
             Field::ExifGpsLongitude => {
-                self.update_file_exif_metadata(entry);
+                self.fms.update_exif_metadata(entry);
 
-                if let Some(ref exif_info) = self.file_exif_metadata {
+                if let Some(ref exif_info) = self.fms.exif_metadata {
                     if let Some(exif_value) = exif_info.get("__Lng") {
                         return Variant::from_float(exif_value.parse().unwrap_or(0.0));
                     }
                 }
             },
             Field::ExifMake => {
-                self.update_file_exif_metadata(entry);
+                self.fms.update_exif_metadata(entry);
 
-                if let Some(ref exif_info) = self.file_exif_metadata {
+                if let Some(ref exif_info) = self.fms.exif_metadata {
                     if let Some(exif_value) = exif_info.get("Make") {
                         return Variant::from_string(&exif_value);
                     }
                 }
             },
             Field::ExifModel => {
-                self.update_file_exif_metadata(entry);
+                self.fms.update_exif_metadata(entry);
 
-                if let Some(ref exif_info) = self.file_exif_metadata {
+                if let Some(ref exif_info) = self.fms.exif_metadata {
                     if let Some(exif_value) = exif_info.get("Model") {
                         return Variant::from_string(&exif_value);
                     }
                 }
             },
             Field::ExifSoftware => {
-                self.update_file_exif_metadata(entry);
+                self.fms.update_exif_metadata(entry);
 
-                if let Some(ref exif_info) = self.file_exif_metadata {
+                if let Some(ref exif_info) = self.fms.exif_metadata {
                     if let Some(exif_value) = exif_info.get("Software") {
                         return Variant::from_string(&exif_value);
                     }
                 }
             },
             Field::ExifVersion => {
-                self.update_file_exif_metadata(entry);
+                self.fms.update_exif_metadata(entry);
 
-                if let Some(ref exif_info) = self.file_exif_metadata {
+                if let Some(ref exif_info) = self.fms.exif_metadata {
                     if let Some(exif_value) = exif_info.get("ExifVersion") {
                         return Variant::from_string(&exif_value);
                     }
                 }
             },
             Field::LineCount => {
-                self.update_file_line_count(entry);
+                self.fms.update_line_count(entry);
 
-                if let Some(line_count) = self.file_line_count {
+                if let Some(line_count) = self.fms.line_count {
                     return Variant::from_int(line_count as i64);
                 }
             },
@@ -1312,9 +1317,9 @@ impl <'a> Searcher<'a> {
                 return Variant::empty(VariantType::String);
             },
             Field::IsBinary => {
-                self.update_file_metadata(entry);
+                self.fms.update_file_metadata(entry, self.current_follow_symlinks);
 
-                if let Some(ref meta) = self.file_metadata {
+                if let Some(ref meta) = self.fms.file_metadata {
                     if meta.is_dir() {
                         return Variant::from_bool(false);
                     }
@@ -1328,9 +1333,9 @@ impl <'a> Searcher<'a> {
                 return Variant::from_bool(false);
             },
             Field::IsText => {
-                self.update_file_metadata(entry);
+                self.fms.update_file_metadata(entry, self.current_follow_symlinks);
 
-                if let Some(ref meta) = self.file_metadata {
+                if let Some(ref meta) = self.fms.file_metadata {
                     if meta.is_dir() {
                         return Variant::from_bool(false);
                     }
@@ -1375,6 +1380,14 @@ impl <'a> Searcher<'a> {
 
                 return Variant::from_bool(is_doc);
             },
+            Field::IsFont => {
+                let is_font = match file_info {
+                    Some(file_info) => self.is_font(&file_info.name),
+                    None => self.is_font(&entry.file_name().to_string_lossy())
+                };
+
+                return Variant::from_bool(is_font);
+            },
             Field::IsImage => {
                 let is_image = match file_info {
                     Some(file_info) => self.is_image(&file_info.name),
@@ -1418,8 +1431,8 @@ impl <'a> Searcher<'a> {
 
     fn check_file(&mut self,
                   entry: &DirEntry,
-                  file_info: &Option<FileInfo>) -> std::io::Result<bool> {
-        self.clear_file_data();
+                  file_info: &Option<FileInfo>) -> io::Result<bool> {
+        self.fms.clear();
 
         if let Some(ref expr) = self.query.expr {
             let result = self.conforms(entry, file_info, expr);
@@ -1446,7 +1459,7 @@ impl <'a> Searcher<'a> {
 
         let mut items: Vec<(String, String)> = Vec::new();
 
-        for field in self.query.fields.clone().iter() {
+        for field in self.query.fields.iter() {
             let record = self.get_column_expr_value(Some(entry), file_info, &mut file_map, None, &field);
 
             let value = match self.use_colors && field.contains_colorized() {
@@ -1465,7 +1478,7 @@ impl <'a> Searcher<'a> {
             }
         }
 
-        for (idx, field) in self.query.ordering_fields.clone().iter().enumerate() {
+        for (idx, field) in self.query.ordering_fields.iter().enumerate() {
             criteria[idx] = match file_map.get(&field.to_string()) {
                 Some(record) => record.clone(),
                 None => self.get_column_expr_value(Some(entry), file_info, &mut file_map, None, &field).to_string()
@@ -1494,7 +1507,7 @@ impl <'a> Searcher<'a> {
     fn colorize(&mut self, value: &str) -> String {
         let style;
 
-        if let Some(ref metadata) = self.file_metadata {
+        if let Some(ref metadata) = self.fms.file_metadata {
             style = self.lscolors.style_for_path_with_metadata(Path::new(&value), Some(metadata));
         } else {
             style = self.lscolors.style_for_path(Path::new(&value));
@@ -1517,9 +1530,9 @@ impl <'a> Searcher<'a> {
                 }
             },
             _ => {
-                self.update_file_metadata(entry);
+                self.fms.update_file_metadata(entry, self.current_follow_symlinks);
 
-                if let Some(ref attrs) = self.file_metadata {
+                if let Some(ref attrs) = self.fms.file_metadata {
                     return Variant::from_bool(mode_func_boxed(attrs));
                 }
             }
@@ -1748,9 +1761,9 @@ impl <'a> Searcher<'a> {
                 },
                 VariantType::DateTime => {
                     let (start, finish) = value.to_datetime();
-                    let start = start.timestamp();
-                    let finish = finish.timestamp();
-                    let dt = field_value.to_datetime().0.timestamp();
+                    let start = start.and_utc().timestamp();
+                    let finish = finish.and_utc().timestamp();
+                    let dt = field_value.to_datetime().0.and_utc().timestamp();
                     match op {
                         Op::Eeq => dt == start,
                         Op::Ene => dt != start,
@@ -1789,6 +1802,10 @@ impl <'a> Searcher<'a> {
         has_extension(file_name, &self.config.is_doc)
     }
 
+    fn is_font(&self, file_name: &str) -> bool {
+        has_extension(file_name, &self.config.is_font)
+    }
+
     fn is_image(&self, file_name: &str) -> bool {
         has_extension(file_name, &self.config.is_image)
     }
