@@ -3,6 +3,7 @@ use std::io;
 mod mkv;
 mod mp3;
 mod mp4;
+mod wav;
 
 use std::path::Path;
 
@@ -11,6 +12,7 @@ use mp3_metadata::MP3Metadata;
 use mkv::MkvDurationExtractor;
 use mp3::Mp3DurationExtractor;
 use mp4::Mp4DurationExtractor;
+use wav::WavDurationExtractor;
 
 #[derive(PartialEq, Eq, Clone, Debug)]
 pub struct Duration {
@@ -22,10 +24,11 @@ pub trait DurationExtractor {
     fn try_read_duration(&self, path: &Path, mp3_metadata: &Option<MP3Metadata>) -> io::Result<Option<Duration>>;
 }
 
-const EXTRACTORS: [&dyn DurationExtractor; 3] = [
+const EXTRACTORS: [&dyn DurationExtractor; 4] = [
     &Mp3DurationExtractor,
     &Mp4DurationExtractor,
     &MkvDurationExtractor,
+    &WavDurationExtractor,
 ];
 
 pub fn get_duration<T: AsRef<Path>>(path: T, mp3_metadata: &Option<MP3Metadata>) -> Option<Duration> {
