@@ -3,6 +3,8 @@ use std::ops::Index;
 use regex::Captures;
 use regex::Regex;
 
+use crate::util::error_exit;
+
 pub fn is_glob(s: &str) -> bool {
     s.contains("*") || s.contains('?')
 }
@@ -21,7 +23,7 @@ pub fn convert_glob_to_pattern(s: &str) -> String {
             ")" => "\\)",
             "^" => "\\^",
             "$" => "\\$",
-            _ => panic!("Error parsing glob")
+            _ => error_exit("Error parsing glob expression", s)
         }.to_string()
     });
 
@@ -44,7 +46,7 @@ pub fn convert_like_to_pattern(s: &str) -> String {
             ")" => "\\)",
             "^" => "\\^",
             "$" => "\\$",
-            _ => panic!("Error parsing like expression")
+            _ => error_exit("Error parsing LIKE expression", s)
         }.to_string()
     });
 
